@@ -39,6 +39,23 @@ class IoTyping(Generic[T]):
     _secure: bool = False
     _recommended: bool = False
     _hdf_key: str = "df"
+    _attrs_suffix: str = ".attrs.json"
+    _use_attrs: bool = False
+
+    @property
+    def use_attrs(self) -> bool:
+        """
+        Whether to read and write ``pd.DataFrame.attrs`` when passing ``attrs=None``.
+        """
+        return self._use_attrs
+
+    @property
+    def attrs_suffix(self) -> str:
+        """
+        File filename suffix detailing where to save/load per-DataFrame "attrs" (metadata).
+        Will be appended to the DataFrame filename.
+        """
+        return self._attrs_suffix
 
     @property
     def hdf_key(self) -> str:
@@ -85,6 +102,9 @@ class IoTyping(Generic[T]):
 
     @property
     def hash_algorithm(self) -> Optional[str]:
+        """
+        The hash algorithm used for checksums.
+        """
         return self._hash_alg
 
     @property
@@ -177,6 +197,13 @@ class DfTyping:
     def io(self) -> IoTyping:
         return self._io_typing
 
+    @property
+    def is_strict(self) -> bool:
+        """
+        Returns True if this allows unspecified index levels **or** columns.
+        """
+        return not self._more_columns_allowed and not self._more_index_names_allowed
+
     @property
     def order_dataclass(self) -> bool:
         """
