@@ -8,6 +8,7 @@ import os
 import sys
 import typing
 from datetime import datetime, timedelta
+from pathlib import Path
 from typing import (
     AbstractSet,
     Any,
@@ -59,6 +60,7 @@ from pandas.io.common import get_handle
 from tabulate import DataRow, TableFormat, _table_formats
 
 from typeddfs._utils import _AUTO_DROPPED_NAMES, _DEFAULT_HASH_ALG, _FORBIDDEN_NAMES
+from typeddfs.df_errors import WritePermissionsError
 from typeddfs.frozen_types import FrozeDict, FrozeList, FrozeSet
 
 T = TypeVar("T")
@@ -132,22 +134,6 @@ class Utils:
         hash(v)  # let it raise an AttributeError
         return v
 
-    @classmethod
-    def natsort(
-        cls,
-        lst: typing.Iterable[T],
-        dtype: Type[T],
-        *,
-        alg: Union[None, int, Set[str]] = None,
-        reverse: bool = False,
-    ) -> Sequence[T]:
-        if alg is None:
-            _, alg = Utils.guess_natsort_alg(dtype)
-        else:
-            _, alg = Utils.exact_natsort_alg(alg)
-        lst = list(lst)
-        return natsorted(lst, alg=alg, reverse=reverse)
-
     @classmethod
     def describe_dtype(cls, t: Type[Any]) -> Optional[str]:
         """
@@ -190,6 +176,48 @@ class Utils:
             return "string"
         return None
 
+    @classmethod
+    def verify_can_write_files(cls, *paths: Union[str, Path], missing_ok: bool = False) -> None:
+        """
+        Checks that all files can be written to, to ensure atomicity before operations.
+
+        Args:
+            *paths: The files
+            missing_ok: Don't raise an error if a path doesn't exist
+
+        Returns:
+            WritePermissionsError: If a path is not a file (modulo existence) or doesn't have 'W' set
+        """
+        paths = [Path(p) for p in paths]
+        for path in paths:
+            if path.exists() and not path.is_file():
+                raise WritePermissionsError(f"Path {path} is not a file", key=str(path))
+            if (not missing_ok or path.exists()) and not os.access(path, os.W_OK):
+                raise WritePermissionsError(f"Cannot write to {path}", key=str(path))
+
+    @classmethod
+    def verify_can_write_dirs(cls, *paths: Union[str, Path], missing_ok: bool = False) -> None:
+        """
+        Checks that all directories can be written to, to ensure atomicity before operations.
+
+        Args:
+            *paths: The directories
+            missing_ok: Don't raise an error if a path doesn't exist
+
+        Returns:
+            WritePermissionsError: If a path is not a directory (modulo existence) or doesn't have 'W' set
+        """
+        paths = [Path(p) for p in paths]
+        for path in paths:
+            if path.exists() and not path.is_dir():
+                raise WritePermissionsError(f"Path {path} is not a dir", key=str(path))
+            if missing_ok and not path.exists():
+                continue
+            if not os.access(path, os.W_OK):
+                raise WritePermissionsError(f"{path} lacks write permission", key=str(path))
+            if not os.access(path, os.X_OK):
+                raise WritePermissionsError(f"{path} lacks access permission", key=str(path))
+
     @classmethod
     def default_hash_algorithm(cls) -> str:
         return _DEFAULT_HASH_ALG
@@ -200,20 +228,32 @@ class Utils:
 
     @classmethod
     def property_key_escape(cls, s: str) -> str:
+        """
+        Escapes a key in a .property file.
+        """
         p = regex.compile(r"([ =:\\])", flags=regex.V1)
         return p.sub(r"\\\1", s)
 
     @classmethod
     def property_key_unescape(cls, s: str) -> str:
+        """
+        Un-escapes a key in a .property file.
+        """
         p = regex.compile(r"\\([ =:\\])", flags=regex.V0)
         return p.sub(r"\1", s)
 
     @classmethod
     def property_value_escape(cls, s: str) -> str:
+        """
+        Escapes a value in a .property file.
+        """
         return s.replace("\\", "\\\\")
 
     @classmethod
     def property_value_unescape(cls, s: str) -> str:
+        """
+        Un-escapes a value in a .property file.
+        """
         return s.replace("\\\\", "\\")
 
     @classmethod
@@ -232,7 +272,8 @@ class Utils:
             dicts: A sequence of dictionaries
 
         Returns:
-            A tomlkit AOT
+            A tomlkit`AoT<https://github.com/sdispater/tomlkit/blob/master/tomlkit/items.py>`_
+            (i.e. ``[[array]]`)
         """
         import tomlkit
 
@@ -251,7 +292,7 @@ class Utils:
         Make sub-dictionaries from substrings in ``items`` delimited by ``.``.
         Used for TOML.
 
-        Examples:
+        Example:
             ``Utils.dots_to_dict({"genus.species": "fruit bat"}) == {"genus": {"species": "fruit bat"}}``
 
         See Also:
@@ -266,7 +307,7 @@ class Utils:
         """
         Performs the inverse of :meth:`dots_to_dict`.
 
-        Examples:
+        Example:
             ``Utils.dict_to_dots({"genus": {"species": "fruit bat"}}) == {"genus.species": "fruit bat"}``
         """
         return dict(cls._re_leaf("", items))
@@ -302,9 +343,9 @@ class Utils:
         Ignores hyphens and lowercases the string.
         Permits these nonstandard shorthands:
 
-          - "platform": use ``sys.getdefaultencoding()`` on the fly
-          - "utf8(bom)": use "utf-8-sig" on Windows; "utf-8" otherwise
-          - "utf16(bom)": use "utf-16-sig" on Windows; "utf-16" otherwise
+          - ``"platform"``: use ``sys.getdefaultencoding()`` on the fly
+          - ``"utf8(bom)"``: use ``"utf-8-sig"`` on Windows; ``"utf-8"`` otherwise
+          - ``"utf16(bom)"``: use ``"utf-16-sig"`` on Windows; ``"utf-16"`` otherwise
         """
         encoding = encoding.lower().replace("-", "")
         if encoding == "platform":
@@ -315,6 +356,35 @@ class Utils:
             encoding = "utf-16-sig" if os.name == "nt" else "utf-16"
         return encoding
 
+    @classmethod
+    def natsort(
+        cls,
+        lst: typing.Iterable[T],
+        dtype: Type[T],
+        *,
+        alg: Union[None, int, Set[str]] = None,
+        reverse: bool = False,
+    ) -> Sequence[T]:
+        """
+        Perform a natural sort consistent with the type ``dtype``.
+        Uses `natsort <https://pypi.org/project/natsort>`_.
+
+        See Also:
+            :meth:`guess_natsort_alg`
+
+        Args:
+            lst: A sequence of things to sort
+            dtype: The type; must be a subclass of each element in ``lst``
+            alg: A specific natsort algorithm or set of flags
+            reverse: Sort in reverse (e.g. Z to A or 9 to 1)
+        """
+        if alg is None:
+            _, alg = Utils.guess_natsort_alg(dtype)
+        else:
+            _, alg = Utils.exact_natsort_alg(alg)
+        lst = list(lst)
+        return natsorted(lst, alg=alg, reverse=reverse)
+
     @classmethod
     def all_natsort_flags(cls) -> Mapping[str, int]:
         """
@@ -330,10 +400,10 @@ class Utils:
         Guesses a good natsorted flag for the dtype.
 
         Here are some specifics:
-            - integers       ==> INT and SIGNED
-            - floating-point ==> FLOAT and SIGNED
-            - strings        ==> COMPATIBILITYNORMALIZE and GROUPLETTERS
-            - datetime       ==> GROUPLETTERS (only affects 'Z' vs. 'z'; shouldn't matter)
+            - integers       ⇒ INT and SIGNED
+            - floating-point ⇒ FLOAT and SIGNED
+            - strings        ⇒ COMPATIBILITYNORMALIZE and GROUPLETTERS
+            - datetime       ⇒ GROUPLETTERS (only affects 'Z' vs. 'z'; shouldn't matter)
 
         Args:
             dtype: Probably from ``pd.Series.dtype``
@@ -363,11 +433,11 @@ class Utils:
         Gets the flag names and combined ``alg=`` argument for natsort.
 
         Examples:
-            - exact_natsort_alg({"REAL"}) == ({"FLOAT", "SIGNED"}, ns.FLOAT | ns.SIGNED)
-            - exact_natsort_alg({}) == ({}, 0)
-            - exact_natsort_alg(ns.LOWERCASEFIRST) == ({"LOWERCASEFIRST"}, ns.LOWERCASEFIRST)
-            - exact_natsort_alg({"localenum", "numafter"})
-              == ({"LOCALENUM", "NUMAFTER"}, ns.LOCALENUM | ns.NUMAFTER)
+            - ``exact_natsort_alg({"REAL"}) == ({"FLOAT", "SIGNED"}, ns.FLOAT | ns.SIGNED)``
+            - ``exact_natsort_alg({}) == ({}, 0)``
+            - ``exact_natsort_alg(ns.LOWERCASEFIRST) == ({"LOWERCASEFIRST"}, ns.LOWERCASEFIRST)``
+            - ``exact_natsort_alg({"localenum", "numafter"})``
+              ``== ({"LOCALENUM", "NUMAFTER"}, ns.LOCALENUM | ns.NUMAFTER)``
 
         Args:
             flags: Can be either:
@@ -407,27 +477,33 @@ class Utils:
     @classmethod
     def table_formats(cls) -> Sequence[str]:
         """
-        Returns the names of styles for :py:mod`tabulate`.
+        Returns the names of styles for `tabulate <https://pypi.org/project/tabulate/>`_.
         """
         return _table_formats.keys()
 
     @classmethod
     def table_format(cls, fmt: str) -> TableFormat:
         """
-        Gets a :py:mod`tabulate` style by name.
+        Gets a tabulate [1]_ style by name.
 
         Returns:
             A TableFormat, which can be passed as a style
+
+        References:
+            [1] `Tabulate <https://pypi.org/project/tabulate>`_
         """
         return _table_formats[fmt]
 
     @classmethod
     def plain_table_format(cls, sep: str = " ", **kwargs) -> TableFormat:
         """
-        Creates a simple :py:mod`tabulate` style using a column-delimiter ``sep``.
+        Creates a simple tabulate [1]_ style using a column-delimiter ``sep``.
 
         Returns:
-            A TableFormat, which can be passed as a style
+            A tabulate ``TableFormat``, which can be passed as a style
+
+        References:
+            [1] `Tabulate <https://pypi.org/project/tabulate>`_
         """
         defaults = dict(
             lineabove=None,
